package SQL::Abstract::Clauses;
use strict;
use base 'SQL::Abstract';
use Carp qw(croak);
use vars qw($VERSION);

$VERSION = '0.02';

=head1 NAME

SQL::Abstract::Clauses - HAVING and GROUP BY clauses for SQL::Abstract

=head1 SYNOPSIS

  my ($sql,@bind) = $s->select($table,[ 'listened', 'count(*) as col_count' ],
                          where => { artist => 'Jamiroquai', mime_type => 'audio/mpeg' },
                          group_by => 'listened',
                          order_by => "col_count desc",
                          having  => { col_count => {'<' => 5 }},
                    );

will generate the SQL statement

  SELECT listened, count(*) as col_count
  FROM files
    WHERE ( artist = ? AND mime_type = ? )
    GROUP BY listened
    HAVING ( col_count < ? )
    ORDER BY col_count desc

and C<@bind> will contain the appropriate bind values:

  # @bind = ('Jamiroquai', 'audio/mpeg', 5)

=head1 CAVEATS

This module provides objects that have a I<slightly different syntax>
from L<SQL::Abstract>. This is the price you pay for wanting
C<HAVING> and C<GROUP BY>.

The C<LIMIT> clause is also recognized, but I haven't thought about
integrating it yet. The module should reuse the work already done
in L<SQL::Abstract::Limit>, but that module extends the
L<SQL::Abstract> syntax in its own way.

=cut

# order taken from SQLite syntax documentation
use vars qw(@clause_order);
@clause_order = qw(where group group_by having order order_by limit);

=head2 C<$sql->having WHERE>

Returns a SQL string and bind parameters containing the appropriate
C<HAVING> clause.

=cut

sub having {
  my ($self,$where) = @_;
  my ($stmt,@bind) = $self->where($where);
  my $having = $self->_sqlcase('having');
  $stmt =~ s!^\s*where\b!$having!ei;
  ($stmt,@bind)
};

=head2 C<< $sql->limit COUNT >>
=head2 C<< $sql->limit [COUNT, OFFSET] >>

Returns a SQL string and bind parameters containing the appropriate
C<LIMIT> clause. You pass in either a single scalar, specifying the
number of rows to return, or an array ref to an array of two
elements.

B<NOT IMPLEMENTED>

=cut

sub limit {
  my ($self,$elt) = @_;
  my ($count,$offset);
  if (ref $elt eq 'ARRAY') {
    ($count,$offset) = @$elt;
  } elsif (not defined ref $elt) {
    $count = $elt
  } else {
    croak "Unknown types passed to LIMIT. Need scalar or arrayref.";
  };

  die "Limiting is not yet implemented.";
};

=head2 C<< $sql->order ORDER >>

ORDER is either a scalar, giving the column (and direction) to order
or an array reference, if you want to order according to more than
a single column.

If you want to order in a certain direction, you need to mash
the direction together with the column name:

  $sql->order( "listened desc" )

This is ugly and will maybe change.

=cut

sub order {
  my ($self,$order) = @_;
  my $res = $self->_order_by($order);
  $res =~ s!^\s*!!;
  $res;
};

=head2 C<< $sql->group GROUP >>

GROUP is either a scalar, giving the column (and direction) to group
by or an array reference, if you want to group by more than
a single column.

=cut

sub group {
  my ($self,$order) = @_;
  my $stmt = $self->_order_by($order);

  my $group_by = $self->_sqlcase('group');
  $stmt =~ s!^\s*order\b!$group_by!ei;

  $stmt
};

=head2 C<< $sql->select TABLE, COLUMNS, %CLAUSES >>

Returns the SQL statement and bind values for the clauses. This
is B<different> from how L<SQL::Abstract> does it. The code
tries to recognize old usage and fall back to SQL::Abstract,
but the fallback doesn't always work.

  my ($sql,@bind) = $s->select('files',[ 'listened', 'count(*) as col_count' ],
                      where => { artist => 'Jamiroquai', mime_type => 'audio/mpeg' },
                      group_by => 'listened',
                      order_by => "col_count desc",
                      having  => { col_count => {'<' => 5 }},

=cut

sub select {
  my $self = shift;
  if (! ref $_[1]) {
    # old style:
    return $self->SUPER::select(@_);
  } else {
    my ($table,$columns,%clauses) = @_;
    my ($sql,@bind) = $self->SUPER::select($table,$columns,delete $clauses{where});
    for my $clause (@clause_order) {
      (my $method = $clause) =~ s!_by$!!;
      if (exists $clauses{$clause}) {
        my ($nsql,@nbind) = $self->$method(delete $clauses{$clause});
        $sql .= " $nsql";
        push @bind, @nbind;
      };
    };
    return wantarray ? ($sql,@bind) : $sql
  };
};

1;

=head1 AUTHOR

Max Maischein, E<lt>corion@cpan.orgE<gt>

=head1 SEE ALSO

L<SQL::Abstract>, L<SQL::Abstract::Limit>